/*
    FEARLUS model0-7-1: MatchWeightedRandomStrategy.m
    Copyright (C) 1999-2003, 2006  Macaulay Institute

    This file is part of FEARLUS model0-7-1, an agent-based model of land use
    change.

    FEARLUS model0-7-1 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FEARLUS model0-7-1 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
/*


Implementation for the MatchWeightedRandomStrategy object.

*/

#import "MatchWeightedRandomStrategy.h"
#import "SelectUseBucket.h"
#import "Environment.h"
#import "LandUse.h"
#import "LandParcel.h"
#import "LandManager.h"
#import "Parameter.h"
#import "FearlusArguments.h"
#import "Interval.h"

@implementation MatchWeightedRandomStrategy

/*

create:withLandManager:

Return a newly created RandomStrategy object, configured with the land manager
specified in the argument

*/

+(id <NonImitativeStrategy, NonHistoricalStrategy>)create: aZone
	   withManager: (LandManager *)lmgr
	 andParameters: (Parameter *)p {
  MatchWeightedRandomStrategy *obj;

  obj = [super create: aZone];
  obj->lm = lmgr;
  obj->parameter = p;
  return obj;
}

/*

decideLandUseForParcel:

Return a land use for the parcel, chosen at random, weighted by bit string
match.

*/

-(LandUse *)decideLandUseForParcel: (LandParcel *)lp {
  id landUses;
  int i;
  SelectUseBucket *bucket;
  LandUse *lu;

  landUses = [[lp getEnvironment] getLandUses];
  bucket = [SelectUseBucket create: [self getZone]
			    withParameters: parameter
			    andLandUses: landUses];
  if([FearlusArguments debugCoreMethods]) {
    printf("S: %s: Making decision at random, weighted by match of land parcel"
	   " with land parcel portion of land use bit string\n",
	   sel_get_name(_cmd));
    printf("\tLand Use   Match with Land Parcel %u at (%d, %d)\n", [lp getPIN],
	   [lp getX], [lp getY]);
  }
  for(i = 0; i < [parameter nUses]; i++) {
    Interval *match = [Interval create];

    lu = [landUses atOffset: i];
    [match set: [Interval castInt: [lu getParcelMatchWith: lp]]];
    [bucket addScore: [match call] toBucketForLandUse: lu];
    if([FearlusArguments debugCoreMethods]) {
      printf("\t%8u   %s\n", [lu getPIN], [match printfVar: "% 8g"]);
      [Interval printfclean];
    }
    [match drop];
  }
  lu = [bucket getChoice];
  [bucket drop];
  return lu;
}

@end
